import argparse


def parse_arguments():
    """
    Parses commandline arguments
    :return: Parsed arguments
    """
    parser = argparse.ArgumentParser('Running in parallel mode')
    parser.add_argument('--env',       '-e',  type=str.lower, help='Environment to use for test run(s)', default=[], nargs='+')
    parser.add_argument('--course',    '-c',  type=str.lower, help='The course [mcat, lsat, etc]')
    parser.add_argument('--processes', '-p',  type=int,       help='Maximum number of processes.', default=1)
    parser.add_argument('--browser',   '-b',  type=str.lower, help='Browser to use')
    parser.add_argument('--headless',  '-hd', help='toggle for headless browser', action='store_true')

    # direct behave arguments
    parser.add_argument('--dir',       '-d',  type=str.lower, help='Path to a behave-like directory')
    parser.add_argument('--itags',     '-it', type=str,       help='Please specify behave tags to run', nargs='+')
    parser.add_argument('--etags',     '-et', type=str,       help='Please specify behave tags to exclude', default=[], nargs='+')
    parser.add_argument('--res',       '-r',  type=str,       help='File path for aggregate results', default='reports/parallel_results/aggregate_results.json')
    parser.add_argument('--retry',     '-rt', type=int,       help='Number of tries')

    # accounts arguments
    parser.add_argument('--accounts_file',    '-a',  type=str,  help='path to accounts ini file')
    parser.add_argument('--accounts_section', '-s',  type=str,  help='path to accounts ini file')

    # syncing arguments
    parser.add_argument('--sync_to', '-st', type=str.lower, default=None)

    # notify arguments
    parser.add_argument('--notify_to', '-nt', type=str.lower, default=None)
    return parser.parse_args()



import configparser
import sys
import os


def get_accounts(
        num_processes: int,
        accounts_file_path: str,
        accounts_section: str
) -> list:
    # get account data available for parallel runs
    data = configparser.ConfigParser()
    data.read(accounts_file_path)
    accounts_data = data[accounts_section]

    # kill everything if more accounts are requested than exists
    if num_processes > len(list(accounts_data.values())):
        print("Not enough accounts; Cannot run tests")
        sys.exit(1)
    else:
        return [acc for acc in list(accounts_data.items())]


import json


def create_aggregate(files: list, aggregate_out_file):
    aggregate = []
    for fp in files:
        with open(fp, 'r') as f:
            aggregate += json.load(f)
    with open(aggregate_out_file, 'w+') as f:
        f.write(json.dumps(aggregate))


def reformat(form: str):
    if form.lower().replace(' ', '').replace('-', '') == 'testrail':
        print('reformat for testrail')
    raise NotImplementedError


def sync(sync_to: str) -> None:
    raise NotImplementedError


# __main__.py
from datetime import datetime
import multiprocessing
from multiprocessing import Pool
import sys

# custom
from . import account_splitter
from . import feature_splitter
from . import arg_parser
from .parallelizer import command_generator
from . import results


def main():
    arguments = arg_parser.parse_arguments()
    # todo - create a temp directory that will store run data and be exported
    # delete old logs, results, and screenshots
    # for directory in ['logs', 'results', 'screenshots']:
    #     _empty_directory(
    #         directory_path=os.path.join('reports', directory),
    #         excluded_file_types=('.gitkeep', '.gitignore',)
    #     )

    # set up the multiple processes # todo - set spawn method based on os (macos -> fork; others -> default)
    multiprocessing.set_start_method('fork')
    pool = Pool(arguments.processes)

    # get account data available for parallel runs
    print('--- Parsing accounts')
    print(f'\tfile:              <{arguments.accounts_file}>')
    print(f'\tsection:           <{arguments.accounts_section}>')
    accounts_data = account_splitter.get_accounts(
        arguments.processes,
        arguments.accounts_file,
        arguments.accounts_section
    )

    # split up the features
    print('--- Grouping features')
    print(f'\tfeature directory: <{arguments.dir}>')
    print(f'\tincluded tags:     <{",".join([t for t in arguments.itags])}>')
    print(f'\texcluded tags:     <{",".join([t for t in arguments.etags])}>')
    account_feature_groups = feature_splitter.get_features(arguments, accounts_data)

    # run all the processes and save the locations of the result files
    print('--- Parallelizing')
    print(f'\tnum processes:     <{len(account_feature_groups)}>')
    res = pool.map(command_generator, account_feature_groups)

    # recombine everything
    print('---RECOMBINING RESULTS')
    try:
        results.create_aggregate(
            files=res,
            aggregate_out_file=arguments.res
        )
    except Exception as e:
        print(e)

    if arguments.sync_to:
        print(f'TODO: implement syncing to {arguments.sync_to}')

    if arguments.notify_to:
        print(f'TODO: implement notifying {arguments.notify_to}')


if __name__ == '__main__':
    args = arg_parser.parse_arguments()
    start = datetime.now()
    main()
    end = datetime.now()
    print('\n========================================')
    print(f'Envs:     <{",".join(args.env)}>')
    print(f'Browser:  <{args.browser}>')
    print(f'Branch:   <figure this out>')
    print(f'Run Time: <{(end - start)}>')
    print('========================================')

    sys.exit(0)
